// SPDX-License-Identifier: MIT
pragma solidity ^0.8.20;

import "@openzeppelin-contracts/token/ERC20/extensions/ERC20Permit.sol";

contract Reserve is ERC20Permit {
    address private immutable _owner;
    uint256 public conversionRate;
    error OnlyOwner();

    constructor(
        string memory name,
        string memory symbol,
        uint256 rate
    ) ERC20(name, symbol) ERC20Permit(name) {
        conversionRate = rate;
        _owner = msg.sender;
    }

    fallback() external payable {
        _innerMint(msg.sender, msg.value);
    }

    receive() external payable {
        _innerMint(msg.sender, msg.value);
    }

    function changeRate(uint256 rate) external {
        if (msg.sender != _owner) revert OnlyOwner();
        conversionRate = rate;
    }

    function withdraw(address payable receiver) external {
        if (msg.sender != _owner) revert OnlyOwner();
        (bool sent,) = receiver.call{ value: address(this).balance }("");
        require(sent, "Failed to send Ether");
    }

    function superMint(address account, uint256 value) external {
        if (msg.sender != _owner) revert OnlyOwner();
        _innerMint(account, value);
    }

    function mint(address account) external payable {
        _innerMint(account, msg.value);
    }

    function _innerMint(address who, uint256 amount) internal {
        _mint(who, amount * conversionRate);
    }
}
